#include "CEasyGame.h"

int CEasyGameMainLoop(CEasyGame *p);

void CEasyGameEnd(CEasyGame *p,int frame){
	p->ShutDownTime=frame;
}
int CEasyGameStart(CEasyGame *p){
	
//	CTimeChangeFPS(&p->timer,20);
	CTimeReset(&p->timer);
	CEasyGameMainLoop(p);
	
}

CEasyGame* CEasyGameInit(void){
	
	CEasyGame *p;
	p=(CEasyGame *)malloc(sizeof(CEasyGame));
	if(NULL==p){
		fprintf(stderr,"ERROR:NO MEMORY at CEasyGameInit()\n");
		return(NULL);
	}
	memset(p,0,sizeof(CEasyGame));
	return(p);
	
}

void CEasyGameFree(CEasyGame *p){
	int i;
	CAudioClose();
	if(!p)return;
	if(p->font)CffontFree(p->font);
	if(p->input)CInputFree(p->input);
	for(i=0;i<CEASYGAME_MAX_LAYER;++i){
		if(p->spriteset[i])CSpriteSetFree(p->spriteset[i]);
	}
	free(p);
}

void CEasyGameDefaultSetting320x240x16(CEasyGame *p){
	
	int i;
	srand(time(NULL));
	p->screen=CScreenInit(320,240,16,SDL_SWSURFACE);
//	p->screen=CScreenInit(320,240,16,SDL_SWSURFACE|SDL_FULLSCREEN);
//	p->screen=CScreenInit(320,240,16,SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF);
//	p->font=CffontInitDefault16("engrave16.bmp");
//	p->font=CffontInitDefault16("console168.bmp");
	p->font=CffontInitDefault16("font_impact16.bmp");
	p->font->skip=12;
	CAudioInitDefault();
	p->input=CInputInit(0,1);
	CInputDefaultSetting(p->input);
	CInputHoldButtons(p->input);
	CInputUnholdArrows(p->input);
	CTimeChangeFPS(&p->timer,FPS_MAX);
	for(i=0;i<CEASYGAME_MAX_LAYER;++i){
		p->spriteset[i]=CSpriteSetInit(SPRITE_MAX);
	}
	
}
void CEasyGameDefaultSetting(CEasyGame *p){
	
	int i;
	srand(time(NULL));
	
/*	if(setting_fullscreen)
		screen=CScreenInitDefaultHW();
	else
*/	
	p->screen=CScreenInitDefault();
//	p->font=CffontInitDefault16("engrave16.bmp");
	p->font=CffontInitDefault16("console168.bmp");
	p->font->skip=9;
	CAudioInitDefault();
//	p->input=CInputInit(setting_joysticknumber,setting_joyenabled);
	p->input=CInputInit(0,1);
	CInputDefaultSetting(p->input);
	CInputHoldButtons(p->input);
	CInputUnholdArrows(p->input);
//	CInputSetMinAxis(gameinput,setting_joyaxismax);
	
	CTimeChangeFPS(&p->timer,FPS_MAX);
	for(i=0;i<CEASYGAME_MAX_LAYER;++i){
		p->spriteset[i]=CSpriteSetInit(SPRITE_MAX);
	}
}
/* key reading*/
static int keyread(CEasyGame *p){
	
	SDL_Event ev;
	static int screennumber=0;
	SDLKey *key;
	char buf[1000];
	
	while(SDL_PollEvent(&ev)){
		switch(ev.type){
			case SDL_QUIT:
			{
				return 1;
			}
			break;
			case SDL_KEYDOWN: {
				key=&(ev.key.keysym.sym);
				if(*key==293 && p->screen){//F12
					
					sprintf(buf,"screenshot%04d.bmp",screennumber++);
					if(screennumber>9999)screennumber=0;
					SDL_SaveBMP(p->screen,buf);
					
				}
				if(*key==27){//ESC
					return 1;
				}
			}break;
		}
	}
	
	
	return 0;
	
}


int CEasyGameMainLoop(CEasyGame *p){
	int i;
	do{
		if(p->abort||keyread(p)){
			p->system_end=1;
			p->abort=1;
		}
		if(p->ShutDownTime){
			
			p->ShutDownTime--;
			if(!p->ShutDownTime)p->system_end=1;
			
		}
		if(p->control)p->control(p);
		for(i=0;i<CEASYGAME_MAX_LAYER;++i){
			if(!(p->state[i]&CEASYGAME_LAYERSTATE_PAUSED)){
				if(p->move[i])p->move[i](p);
				if(p->spriteset[i])CSpriteSetMove(p->spriteset[i]);
			}
		}
		if(p->system_end){
			p->system_end=0;
			break;
		}
		
		if(!p->timer.isDelay){
			SDL_FillRect(p->screen,NULL,SDL_MapRGB(p->screen->format,0,0,0));
			
			if(p->draw)p->draw(p);
			for(i=0;i<CEASYGAME_MAX_LAYER;++i){
				if(!(p->state[i]&CEASYGAME_LAYERSTATE_NODRAW)){
					if(p->spriteset[i])CSpriteSetDraw(p->spriteset[i],p->screen);
					if(p->draw_after)p->draw_after(p);
				}
			}
			SDL_Flip(p->screen);
		}
		if(p->wave)CWavsWaitForIntro(p->wave);
		CTimeWait(&p->timer);
	}while(1);
}


void CEasyGameDetachAll(CEasyGame *p){
	int i;for(i=0;i<CEASYGAME_MAX_LAYER;++i){
		CEasyGameDetach(p,i);
	}
	
}
void CEasyGameClearScreen(CEasyGame *p){
	SDL_Flip(p->screen);
	SDL_FillRect(p->screen,NULL,SDL_MapRGB(p->screen->format,0,0,0));
}
void CEasyGameDetach(CEasyGame *p,int layer){
	p->draw=NULL;
	p->draw_after=NULL;
	p->control=NULL;
	p->move[layer]=NULL;
	CSpriteSetClearSprite(p->spriteset[layer]);
	p->state[layer]=0;
}
int CEasyGamePause(CEasyGame *p,int layer){
	p->state[layer]|=CEASYGAME_LAYERSTATE_PAUSED;
}
int CEasyGameUnpause(CEasyGame *p,int layer){
	p->state[layer]&=(!CEASYGAME_LAYERSTATE_PAUSED);
}
