/**************
simple map funcs
2000/1/22
**************/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "CMap.h"

CMap* CMapInit(int size){
	int i,j;
	CMap *this;
	
	this=(CMap *)malloc(sizeof(CMap));
	if(!this){
		fprintf(stderr,"out of memory at CMapInit");
		return NULL;
	}
	
	this->size=size;
	
	this->key=(unsigned char **)malloc(sizeof(unsigned char *)*size);
	if(this->key==NULL)goto error1;
	
	this->value=(unsigned char **)malloc(sizeof(unsigned char *)*size);
	if(this->value==NULL)goto error2;
	
	for(i=0;i<size;++i){
		this->key[i]=(unsigned char *)malloc(sizeof(unsigned char)*KEY_SIZE);
		if(this->key[i]==NULL) goto error3;
	}
	for(j=0;j<size;++j){
		this->value[j]=(unsigned char *)malloc(sizeof(unsigned char)*VALUE_SIZE);
		if(this->value[j]==NULL) goto error4;
	}
	CMapClear(this);
	
	return this;
	
	
error4:
	for(;j>0;--j)
		free(this->value[j-1]);
error3:
	for(;i>0;--i)
		free(this->key[i-1]);
	free(this->value);
error2:
	free(this->key);
error1:
	free(this);
	return NULL;
	
}

void CMapFree(CMap *this){
	int i,j;
	if(!this)return;
	for(j=this->size;j>0;--j)
		free(this->value[j-1]);
		
	for(i=this->size;i>0;--i)
		free(this->key[i-1]);
		
	free(this->value);
	free(this->key);
	free(this);
	return;
}

//return index of key.if no entry,return -1
int CMapGetIndex(CMap *this,unsigned char *key){
	int i;
	for(i=0;i<this->size;i++){
		if(strcmp(this->key[i],key)==0){
			return(i);
		}
	}
	return(-1);
}

//search available index.if not,return -1
int CMapGetEntryNumber(CMap *this){
	int i;

	for(i=0;i<this->size;i++){
		if(this->key[i][0] == '\0'){
			return(i);
		}
	}
	return(-1);
}

unsigned char *CMapGetValue(CMap *this,unsigned char *key){
	int i;
	i=CMapGetIndex(this,key);
	if(i==-1)return NULL;
	return(this->value[i]);
}

int CMapAdd(CMap *this,unsigned char *key,unsigned char *val){
	
	int i=0;
	if(!key){
		fprintf(stderr,"adding Null keyword at CMap!");
		return -1;
	}
	i=CMapGetIndex(this,key);
	if(i==-1){
		i=CMapGetEntryNumber(this);
		
		if(i==-1){
			fprintf(stderr,"over flow at CMap.\n");
			return(-1);
		}
	}
	
	strcpy(this->key[i],key);
	if(val!=NULL)strcpy(this->value[i],val);else *(this->value[i])=0;
	return(0);
}

int CMapClear(CMap *this){
	int i;
	for(i=0;i<this->size;i++){
		this->key[i][0]='\0';
		this->value[i][0]='\0';
	}
	return(0);
}


int CMapPrint(CMap *this){
	int i;
	printf("------CMap-----\n");
	for(i=0;i<this->size;++i){
		if(this->key[i][0]!='\0')printf("%s %s\n",this->key[i],this->value[i]);
	}
	return(0);
}



int CMapLoadFromFile(CMap *this,char *filename){
	int i;
	char buf[4096];
	char *ignore="#\r\n";
	char *p ,*q;
	FILE *fp;
	
	fp=fopen(filename,"rb");
	if(!fp){
		fprintf(stderr,"ERROR:cannot open %s by CMap\n",filename);
		return -1;
	}
	
	do{
		if(NULL==(fgets(buf,4096,fp)))break;
		if(NULL!=strchr(ignore,*buf))continue;
		
		p=strtok(buf,"\r\t\n #");
		q=strtok(NULL,"\r\t\n #");
		
		CMapAdd(this,p,q);
		
	}while(1);
	fclose(fp);
	
	
	return(0);
}
int CMapSaveToFile(CMap *this,char *filename){
	int i;
	FILE *fp;
	
	fp=fopen(filename,"wb");
	if(!fp){
		fprintf(stderr,"cannot create %s\n",filename);
		return -1;
	}
	fprintf(fp,"# this is generated by CMap automatically\n");
	for(i=0;i<this->size;i++){
		if(!this->key[i][0])continue;
		fprintf(fp,"%s\t\t\t\t%s\n",this->key[i],this->value[i]);
	}
	
	fclose(fp);
	
	
	return(0);
}
