#include "CSprite.h"
CSpriteSet* CSpriteSetInit(int n){
	CSpriteSet *p;
	if( n < 0 )return(NULL);
	p=(CSpriteSet *)malloc(sizeof(CSpriteSet));
	if(NULL==p){
		fprintf(stderr,"ERROR:NO MEMORY at CSpriteSetInit()\n");
		return(NULL);
	}
	memset(p,0,sizeof(CSpriteSet));
	p->n_sprite=n;
	p->sprite=(TSprite *)malloc(sizeof(TSprite)*p->n_sprite);
	if(p->sprite == NULL){
		fprintf(stderr,"ERROR:NO MEMORY at CSpriteSetInit()\n");
		free(p);
		return(NULL);
	}
	memset(p->sprite,0,sizeof(TSprite *)*p->n_sprite);
	return(p);
}
void CSpriteSetFree(CSpriteSet *this){
	
	if(this==NULL)return;
	
	if(NULL==this->sprite)	free(this->sprite);
	free(this);
	
	return;
}
TSprite* CSpriteSetSetWizNum(CSpriteSet *this,CBmps *graphix,int number){
	
	TSprite *sp;
	int i;
	int j;
	i=number;
	j=i;
	do{
		i++;i%=this->n_sprite;
		sp=&this->sprite[i];
		if(!sp->isExist){
			
			memset(sp,0,sizeof(TSprite));
			
			sp->x=0;
			sp->y=0;
			sp->graphix=graphix;
			sp->isExist=1;
			sp->prop.anime_index=-1;
			sp->prop.anime[0].pattern=SPRITE_ENDSWITCH_STOP;
			sp->alpha=SDL_ALPHA_OPAQUE;
			sp->type=TSPRITE_GRAPHIC;
			return sp;
		}
		
	}while(i!=j);
//	if(i==j)return -1;
	return NULL;
	
}
TSprite* CSpriteSetTextSetWizNum(CSpriteSet *this,Cffont *font,int number){
	
	TSprite *sp;
	int i;
	int j;
	i=number;
	j=i;
	do{
		i++;i%=this->n_sprite;
		sp=&this->sprite[i];
		if(!sp->isExist){
			
			memset(sp,0,sizeof(TSprite));
			
			sp->x=0;
			sp->y=0;
			sp->font=font;
			sp->isExist=1;
			sp->prop.anime_index=-1;
			sp->prop.anime[0].pattern=SPRITE_ENDSWITCH_STOP;
			sp->alpha=SDL_ALPHA_OPAQUE;
			sp->type=TSPRITE_TEXT;
			return sp;
		}
		
	}while(i!=j);
//	if(i==j)return -1;
	return NULL;
	
}
TSprite* CSpriteSetSurfaceSetWizNum(CSpriteSet *this,SDL_Surface *surface,int number){
	
	TSprite *sp;
	int i;
	int j;
	i=number;
	j=i;
	do{
		i++;i%=this->n_sprite;
		sp=&this->sprite[i];
		if(!sp->isExist){
			
			memset(sp,0,sizeof(TSprite));
			
			sp->x=0;
			sp->y=0;
			sp->surface=surface;
			sp->isExist=1;
			sp->prop.anime_index=-1;
			sp->prop.anime[0].pattern=SPRITE_ENDSWITCH_STOP;
			sp->alpha=SDL_ALPHA_OPAQUE;
			sp->type=TSPRITE_SDLSURFACE;
			return sp;
		}
		
	}while(i!=j);
//	if(i==j)return -1;
	return NULL;
	
}
void CSpriteSetUnset(CSpriteSet *this,TSprite *sprite){
	
	TSprite *sp;
	int i;
	for(i=0;i<this->n_sprite;++i){
		sp=&this->sprite[i];
		if(sp==sprite){
			if(sp->type==TSPRITE_SDLSURFACE){
				SDL_FreeSurface(sp->surface);
			}
			memset(sp,0,sizeof(TSprite));
		}
	}
	return;
	
}
void CSpriteSetFadeOut(CSpriteSet *this,int frame){
	
	TSprite *sp;
	int i;
	for(i=0;i<this->n_sprite;++i){
		sp=&this->sprite[i];
		if(sp->isExist){
			TSpriteSetFadeOutExtention(sp,frame);
		}
	}
	return;
	
}
void CSpriteSetFadeIn(CSpriteSet *this,int frame){
	
	TSprite *sp;
	int i;
	for(i=0;i<this->n_sprite;++i){
		sp=&this->sprite[i];
		if(sp->isExist){
			TSpriteSetFadeInExtention(sp,frame);
		}
	}
	return;
	
}
void CSpriteSetDraw(CSpriteSet *this,SDL_Surface *screen){
	
	int i;
	TSprite *sp;
	Sint32 x,y;
	SDL_Rect d;
	
	
	for(i=0;i<this->n_sprite;++i){
		
		sp=&this->sprite[i];
		if(!sp->isExist || (!sp->graphix && !sp->font && !sp->surface)|| !sp->isVisible || sp->wait)continue;
		
		
		x=sp->x;y=sp->y;
		if(sp->xystyle==1){
			x-=this->delta_x;
			y-=this->delta_y;
		}
		switch(sp->type){
			case TSPRITE_GRAPHIC:
				if(sp->alpha==SDL_ALPHA_OPAQUE)
					CBmpsBlit(sp->graphix,screen,sp->number,x>>UNDER_PERIOD,y>>UNDER_PERIOD);
				else
					CBmpsBlitAlpha(sp->graphix,screen,sp->number,x>>UNDER_PERIOD,y>>UNDER_PERIOD,sp->alpha);
			break;
			
			case TSPRITE_TEXT:
				if(sp->alpha==SDL_ALPHA_OPAQUE)
				CffontBlitxy(sp->font,sp->text,screen,x>>UNDER_PERIOD,y>>UNDER_PERIOD);
				else
				CffontBlitxyAlpha(sp->font,sp->text,screen,x>>UNDER_PERIOD,y>>UNDER_PERIOD,sp->alpha);
			
			
			break;
			
			case TSPRITE_SDLSURFACE:
				d.x=x>>UNDER_PERIOD;
				d.y=y>>UNDER_PERIOD;
				d.w=sp->surface->w;
				d.h=sp->surface->h;
				if(sp->alpha==SDL_ALPHA_OPAQUE)
					SDL_BlitSurface(sp->surface,NULL,screen,&d);
				else{
					SDL_SetAlpha(sp->surface,SDL_SRCALPHA,sp->alpha);
					SDL_BlitSurface(sp->surface,NULL,screen,&d);
					SDL_SetAlpha(sp->surface,0,sp->alpha);
				}
			
			break;
		}	
	}
	return;
}


void CSpriteSetMove(CSpriteSet *this){
	
	int i;
	TSprite *sp;
	TSpriteSubProperty *prop;
	
	for(i=0;i<this->n_sprite;++i){
		
		sp=&this->sprite[i];
		prop=&sp->prop;
		if(!sp->isExist)continue;
		if(sp->wait){
			
			sp->wait--;
			continue;
			
		}
		//
		if(prop->countdown){
			
			prop->countdown--;
			if(!prop->countdown){
				
				TSpriteDetach(sp);
				
				continue;
			}
			
		}
		//~
		if(prop->stopcountdown){
			
			prop->stopcountdown--;
			if(!prop->stopcountdown){
				
				prop->dx=0;
				prop->dy=0;
				
				continue;
			}
			
		}
		switch(sp->ex.type){
			case TSPRITEEX_SHADOW:
			if(!TSpriteIsMove(sp)){
				break;
			}
			
			sp->ex.frame++;
			sp->ex.frame%=sp->ex.max_frame;
			if(!sp->ex.frame){
				TSprite *shadow;
				if(sp->graphix){
#define SHADOW_INDEX	1800
					shadow= CSpriteSetSetWizNum(this,sp->graphix,SHADOW_INDEX);
					shadow->number=sp->number;
				}else if(sp->font){
					
					shadow=CSpriteSetTextSetWizNum(this,sp->font,SHADOW_INDEX);
					strcpy(shadow->text,sp->text);
				}
				shadow->alpha=255;
				shadow->isVisible=1;
				shadow->x=sp->x;
				shadow->y=sp->y;
				shadow->xystyle=sp->xystyle;
				shadow->prop.countdown=10;
				TSpriteSetFadeOutExtention(shadow,20);
				
			}
			
			break;
			case TSPRITEEX_BLINK:
			sp->ex.frame++;
			sp->ex.frame%=sp->ex.max_frame;
			if(sp->ex.frame<sp->ex.max_frame/2)sp->isVisible=1;else sp->isVisible=0;
			break;
			case TSPRITEEX_FADEOUT:
			sp->ex.frame++;
			if(sp->ex.max_frame==sp->ex.frame){
				TSpriteDetach(sp);
			}else{
				sp->alpha=255*(sp->ex.max_frame-sp->ex.frame)/sp->ex.max_frame;
			}
			break;
			case TSPRITEEX_FADEIN:
			sp->ex.frame++;
			if(sp->ex.max_frame==sp->ex.frame){
				TSpriteUnsetExtention(sp);
			}else{
				sp->alpha=255*sp->ex.frame/sp->ex.max_frame;
			}
			break;
		}
		if(sp->motion){
			sp->motion(sp);
		}else{
			sp->x+=sp->prop.dx;
			sp->y+=sp->prop.dy;
			
			//Aj
			if(prop->anime_index>=0){
				
				prop->frame++;
				if(prop->frame>prop->anime[prop->anime_index].waitframe){
					prop->frame=0;
					prop->anime_index++;
				}
				switch(prop->anime[prop->anime_index].pattern){
					
					case SPRITE_ENDSWITCH_OUT:
					
					TSpriteDetach(sp);
					continue;
					break;
					
					
					case SPRITE_ENDSWITCH_STOP:
					
					sp->number=prop->anime[prop->anime_index-1].pattern;
					TSpriteUnsetAnime(sp);
					
					break;
					
					case SPRITE_ENDSWITCH_LOOP:
					
					prop->anime_index=0;
					sp->number=prop->anime[0].pattern;
					prop->frame=0;
					
					break;
					
					default:
					
					sp->number=prop->anime[prop->anime_index].pattern;
					
					break;
					
				}//end of switch
				
			}
			
			
		}
	}
	return;
}


void CSpriteSetClearSprite(CSpriteSet *this){
	int i;
	TSprite *sp;
	if(!this)return;
	for(i=0;i<this->n_sprite;++i){
		sp=&this->sprite[i];
		CSpriteSetUnset(this,sp);
	}
	
}





int TSpriteDetach(TSprite *sp){
	
	memset(sp,0,sizeof(TSprite));
	TSpriteUnsetAnime(sp);
	
}



int TSpriteSetPos(TSprite *sp,int x,int y){
	
	sp->x=x*(1<<UNDER_PERIOD);
	sp->y=y*(1<<UNDER_PERIOD);
	
	sp->prop.dx=0;
	sp->prop.dy=0;
	return;
	
};
int TSpriteIsMove(TSprite *sp){
	
	if(sp->prop.dx||sp->prop.dy)return 1;else return 0;
	
};
int TSpriteSetGrphicnumber(TSprite *sp,int n){
	
	sp->number=n;
	return;
	
};
int TSpriteSetMotionLiner(TSprite *sp,int sx,int sy,int dx,int dy,int frame){
	
	
	sp->x=sx<<UNDER_PERIOD;
	sp->y=sy<<UNDER_PERIOD;
	if(sx<dx){
		sp->prop.dx=((dx-sx)<<UNDER_PERIOD)/frame;
	}else{
		sp->prop.dx=-((sx-dx)<<UNDER_PERIOD)/frame;
	}
	if(sy<dy){
		sp->prop.dy=((dy-sy)<<UNDER_PERIOD)/frame;
	}else{
		sp->prop.dy=-((sy-dy)<<UNDER_PERIOD)/frame;
	}
	sp->prop.stopcountdown=frame+1;
	return;
	
};


int TSpriteSetMotionBilinerImpliment(TSprite *sp){
	
	sp->x=(sp->x+2*sp->prop.destx)/3;
	sp->y=(sp->y+2*sp->prop.desty)/3;
	sp->prop.frame--;
	if(sp->prop.frame==0)sp->motion=NULL;
}
int TSpriteSetMotionBiliner(TSprite *sp,int sx,int sy,int dx,int dy,int frame){
	
	
	sp->motion=TSpriteSetMotionBilinerImpliment;
	sp->x=sx<<UNDER_PERIOD;
	sp->y=sy<<UNDER_PERIOD;
	sp->prop.destx=dx<<UNDER_PERIOD;
	sp->prop.desty=dy<<UNDER_PERIOD;
	sp->prop.dx=0;
	sp->prop.dy=0;
	sp->prop.frame=frame;
	return;
	
};


int TSpriteSetMotionLinerArrangeImpliment(TSprite *sp){
	
	sp->prop.frame--;
	if(sp->prop.frame==0){
		sp->motion=NULL;
		sp->prop.dx=0;
		sp->prop.dy=0;
	}
	if(sp->prop.frame==5+10){
		sp->prop.dx/=5;
		sp->prop.dy/=5;
	}
	if(sp->prop.frame==10){
		sp->prop.dx/=2;
		sp->prop.dy/=2;
	}
	sp->x+=sp->prop.dx;
	sp->y+=sp->prop.dy;
}
int TSpriteSetMotionLinerArrange(TSprite *sp,int sx,int sy,int dx,int dy,int frame){
	
	sp->motion=TSpriteSetMotionLinerArrangeImpliment;
	TSpriteSetMotionLiner(sp,sx,sy,dx,dy,frame);
	sp->prop.frame=frame-1+5+10;
	sp->prop.countdown=0;
	return;
	
};




int TSpriteUnsetAnime(TSprite *sp){
	sp->prop.anime_index=-1;
	sp->prop.anime[0].pattern=SPRITE_ENDSWITCH_STOP;
	
}
int TSpriteSetAnime(TSprite *sp,int number,int frame,int endswitch){
	
	int set_index,i;
	set_index=ANIME_MAX;//󂫂ꍇAŌ̃Ajp^[ύX
	for(i=0;i<ANIME_MAX;++i){
		if(sp->prop.anime[i].pattern==SPRITE_ENDSWITCH_LOOP||
			sp->prop.anime[i].pattern==SPRITE_ENDSWITCH_STOP){set_index=i;break;}
	}
	sp->prop.anime_index=0;
	sp->prop.frame=0;
	sp->prop.anime[set_index].pattern=number;
	sp->prop.anime[set_index].waitframe=frame;
	sp->prop.anime[set_index+1].pattern=endswitch;
}


int TSpriteSetShadowExtention(TSprite *sp){
	sp->ex.type=TSPRITEEX_SHADOW;
	sp->ex.max_frame=3;
}
int TSpriteUnetShadowExtention(TSprite *sp){
	memset(&sp->ex,0,sizeof(TSpriteExtention));
	
}
int TSpriteSetBlinkExtention(TSprite *sp,int interval){
	sp->ex.type=TSPRITEEX_BLINK;
	sp->ex.max_frame=interval;
}
int TSpriteUnsetExtention(TSprite *sp){
	memset(&sp->ex,0,sizeof(TSpriteExtention));
	
}
int TSpriteSetFadeOutExtention(TSprite *sp,int interval){
	sp->ex.type=TSPRITEEX_FADEOUT;
	sp->ex.max_frame=interval;
}
int TSpriteSetFadeInExtention(TSprite *sp,int interval){
	sp->alpha=0;
	sp->ex.type=TSPRITEEX_FADEIN;
	sp->ex.max_frame=interval;
}



